# vim: et sw=4

import os, subprocess, time, optparse

def createcontrolfile(path, packagename, version, maintainer='Arnold Krille <arnold@arnoldarts.de>', description='', additional_args={}):
    fullpath = os.path.join(path,packagename,'DEBIAN');
    if not os.path.isdir(fullpath):
        os.makedirs(fullpath)
    f = open(os.path.join(fullpath,'control'), 'w')
    control = """\
Package: #PACKAGENAME#
Version: #VERSION#
Architecture: #ARCH#
Maintainer: #MAINTAINER#
"""
    control = control.replace('#PACKAGENAME#', packagename)
    control = control.replace('#VERSION#', version)
    control = control.replace('#MAINTAINER#', maintainer)
    if 'depends' in additional_args: control += 'Depends: %s\n' % additional_args['depends']
    if 'section' in additional_args: control += 'Section: %s\n' % additional_args['section']
    if 'arch' in additional_args:
        arch = additional_args['arch']
    else:
        arch = 'all'
    control = control.replace('#ARCH#', arch)
    control += "Description: %s\n\n" % description
    f.write(control)
    f.close()

def getversion(cwd='.'):
    process = subprocess.Popen(['svnversion'], stdout=subprocess.PIPE, cwd=cwd)
    process.wait()
    version = '0.6~svn' + process.stdout.readline().split(':')[-1].strip()
    if version[-1] is 'M':
        version += time.strftime('-%Y%m%d-%H%M')
    return version

def getarch():
    process = subprocess.Popen(['uname', '-m'], stdout=subprocess.PIPE)
    process.wait()
    arch = process.stdout.readline().strip()
    if arch == 'x86_64':
        arch = 'amd64'
    return arch

def buildpackage(package, buildtarget, dependencies, section, description, cwd, debsdir, version=None, arch=getarch()):
    if not os.path.isdir(os.path.join(cwd, 'debs')):
        os.makedirs(os.path.join(cwd, 'debs'))

    if version is None:
        version = getversion(cwd)
    #print 'buildpackage: ', version

    subprocess.call(['scons', 'PREFIX=/usr', 'DESTDIR=#debs/%s' % package, buildtarget, '-s', ], cwd=cwd)

    createcontrolfile(os.path.join(cwd, 'debs'), package, version=version, description=description, additional_args={'depends': dependencies.replace('VERSION', version), 'section': section, 'arch': arch, })

    subprocess.call(['dpkg-deb', '-b', os.path.join(cwd, 'debs', package), debsdir])

    return '%s_%s_%s.deb' % (package, version, arch)


def buildpackages(packages, cwd, debsdir, version=None, arch=getarch()):
    if version is None:
        version = getversion(cwd)
    #print 'buildpackages: ', version
    ret = []
    for package, buildtarget, dependencies, section, description in packages:
        ret.append( buildpackage(package, buildtarget, dependencies, section, description, cwd, debsdir, version, arch) )
    return ret

def main(packages, version=None):
    print "Doing an automated build..."

    parser = optparse.OptionParser()#usage='usage: %prog [options]' )
    parser.add_option('-t', '--target', dest='targetdir', help='Save the packages into the directory DIR', metavar='DIR', default='debs')
    parser.add_option('-d', '--workingdir', dest='cwd', help='Working directory where the build is done', metavar='DIR', default=os.sep.join(__file__.split(os.sep)[:-2]))

    (options, args) = parser.parse_args()

    #print options
    #print args
    #print __file__, __name__

    return buildpackages(packages, options.cwd, options.targetdir)
