#!/usr/bin/python

import re
import os.path

include_re = re.compile(r'\n\#include \"([a-zA-Z0-9._\-]*)\"', re.M)

import SCons.Builder
import SCons.Tool

def exists( env ):
	return True

moc = SCons.Builder.Builder( action="moc $SOURCES > $TARGET", suffix='.moc', src_suffix='.h'   )
uic = SCons.Builder.Builder( action="uic $SOURCES > $TARGET", suffix='.h',   src_suffix='.ui'  )
rcc = SCons.Builder.Builder( action="rcc -name $TARGET $SOURCES > $TARGET", suffix='.cpp', src_suffix=".qrc" )

def xml2proxyaction( target, source, env ):
	dir = "/".join( str(source[0]).split("/")[0:-1] )
	source = str(source[0]).split("/")[-1]
	target = str(target[0]).split("/")[-1]
	prefix = ".".join( target.split(".")[0:-1] )
	action = "cd %s && qdbusxml2cpp -m -p %s %s" % ( dir, prefix, source )
	env.Execute( action )
	return 0
def xml2adaptoraction( target, source, env ):
	dir = "/".join( str(source[0]).split("/")[0:-1] )
	source = str(source[0]).split("/")[-1]
	target = str(target[0]).split("/")[-1]
	prefix = ".".join( target.split(".")[0:-1] )
	action = "cd %s && qdbusxml2cpp -m -a %s %s" % ( dir, prefix, source )
	env.Execute( action )
	return 0

def xml2proxyemitter( target, source, env ):
	#print "xml2proxyemitter( " + str(target[0]) + ", " + str(source[0]) + ", env)"
	return ( target + [".".join(str(target[0]).split(".")[0:-1]) + ".h" ], source )

xml2proxy = SCons.Builder.Builder( action=xml2proxyaction, suffix=".cpp", src_suffix=".xml", emitter=xml2proxyemitter )
xml2adaptor = SCons.Builder.Builder( action=xml2adaptoraction, suffix=".cpp", src_suffix=".xml", emitter=xml2proxyemitter )

def uic_scanner( node, env, path ):
	#print "uic_scanner: " + str(node)
	if str(node).split(".")[-1] == "moc":
		#print "mocbuilder"
		header = str(node).split(".")[0:-1][0] + ".h"
		env.MOCBuilder( source=header, target=str(node) )
		return [ str(node).split("/")[-1].split(".")[0:-1][0] + ".h" ]

	contents = node.get_contents()
	includes = include_re.findall(contents)
	if str(node).split(".")[-1] == "cpp" or str(node).split(".")[-1] == "h":
		#print "uic_scanner processing a cpp-file " + str(node)
		ret = []
		for inc in includes:
			header_to_check = os.path.join(str(node.dir),inc)
			#print "Header to check: " + header_to_check

			if os.path.exists( os.path.join( str(node.dir), inc ) ) and inc.endswith( ".h" ):
				#print " Found a normal include: " + inc
				ret.append( inc )
			if inc.endswith( ".moc" ):
				#print " Found a moc include: " + inc
				ret.append( inc )

			ui_to_check = header_to_check.split(".")[0:-1][0] +".ui"
			#print "Ui to check: " + ui_to_check
			if os.path.exists( ui_to_check ):
				#print "Header doesn't exist but .ui file does!"
				env.UIBuilder( target = header_to_check, source = ui_to_check )
				ret.append( inc )

			xml_to_check = header_to_check.split(".")[0:-1][0]
			#print xml_to_check
			if xml_to_check.count("_") > 0:
				#print "2: %s" % xml_to_check.split( "_" )[0:-1]
				xml_to_check = "-".join( xml_to_check.split("_")[0:-1] )
			xml_to_check += ".xml"
			#print "xml to check: %s (from %s)" % (xml_to_check, header_to_check)
			if os.path.exists( xml_to_check ):
				#print "Header doesn't exist but .xml file does!"
				if header_to_check.count("_adaptor") > 0:
					#print "Will run adaptor-action for %s" % header_to_check
					env.DBusXml2AdaptorBuilder( target= ".".join( header_to_check.split(".")[0:-1] ), source = xml_to_check )
				else:
					env.DBusXml2ProxyBuilder( target = ".".join(header_to_check.split(".")[0:-1]), source = xml_to_check )
				ret.append( inc )

			if os.path.exists( header_to_check + ".in" ):
				ret.append( inc )

		#print "uic_scanner returning" + str(ret)
		return ret

	print "\nWarning! was called with an unhandled suffix: " + str(node).split(".")[-1] + " !\n"
	return []


def generate( env ):
	#print "qt4muc"

	env['BUILDERS']['UIBuilder'] = uic
	env['BUILDERS']['MOCBuilder'] = moc
	env['BUILDERS']['RCCBUILDER'] = rcc
	env['BUILDERS']['DBusXml2ProxyBuilder'] = xml2proxy
	env['BUILDERS']['DBusXml2AdaptorBuilder'] = xml2adaptor

	SCons.Tool.SourceFileScanner.add_scanner( '.cpp', SCons.Scanner.Scanner( uic_scanner, skeys=[".h"], recursive=1 ) )

