/*
    Copyright 2008-2010 EPOS-Project
                        (FZ Dresden-Rossendorf & MLU Halle-Wittenberg)
                        Arnold Krille <arnold@arnoldarts.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 3 of the License.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; see the file gpl-3.0.txt. If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "emea.h"

#include <QtGui/QApplication>
#include <QtCore/QDebug>

#include <acqiris.h>
#include <resulttospectrum.h>
#include <evaluationpath.h>
#include <compactcsv.h>

class EMEAAcqiris : public EMEABase
{
	public:
		EMEAAcqiris( QObject* p ) : EMEABase( p ) {
			qDebug() << "EMEAAcqiris::EMEAAcqiris(" << p << ")";
		}

		void setupChain( int threads ) {
			qDebug() << "EMEAAcqiris::setupChain(" << threads << ")";
			acqiris = objectInNewThread<NSAcqiris::Acqiris>( true );
			for ( int i=0; i<acqiris->digitizers(); ++i ) {
				qDebug() << i << "got digitizer" << acqiris->digitizer( i );
				addToChainStart( acqiris->digitizer( i ) );
				acqiris->digitizer( i )->setDispatching( NSCore::DataSetProducer::DispatchEqual );
			}
			QList<NSCore::DataSetChainMember*> evals;
			for ( int i=0; i<threads; ++i )
				evals << new NSEvaluation::EvaluationPath( this );
			addToChain( evals );
			evals.clear();
			NSSpectrum::ResultsToSpectrum* res = new NSSpectrum::ResultsToSpectrum( this );
			connect( res, SIGNAL( newSpectrum( NSSpectrum::AbstractSpectrumModel* ) ),
					this, SIGNAL( newSpectrum( NSSpectrum::AbstractSpectrumModel* ) ) );
			addToChain( res->chainmembers(), false );
			addToGui( res );
			addToChain( writer = new NSDiskIO::CompactCSVWriter( this ) );
		}

		NSAcqiris::Acqiris* acqiris;
		NSDiskIO::AbstractFileWriter* writer;

		void autoStart() {
			qDebug() << "EMEAAcqiris::autoStart()";
			for ( int i=0; i<acqiris->digitizers(); ++i )
				acqiris->digitizer( i )->start();
		}
};

EMEA_MAIN( EMEAAcqiris );

